package com.example.vocab.basic


open class Student: StudentSync, Syncable{
    var id: String
        protected set
    var class_id: String
        protected set
    var name: String
        protected set
    var listWords: MutableList<Word>
        protected set
    var listWebs: MutableList<Web>
        protected set
    var listHomework: MutableList<HomeWork>
        protected set
    var listTests = mutableListOf<Test>()
        protected set

    protected constructor(): super(){
        id = ""
        class_id = ""
        name = ""
        listWords = mutableListOf()
        listWebs = mutableListOf()
        listHomework = mutableListOf()
    }

    constructor(id: String, name: String, listWords : MutableList<Word>, listWebs: MutableList<Web>,
                listHomework: MutableList<HomeWork>, class_id: String, new: Boolean = false,
                tests: MutableList<Test>? = null): super() {

        this.id = id
        this.name = name
        this.listWords = listWords
        this.listWebs = listWebs
        this.listHomework = listHomework
        this.class_id = class_id

        tests?.let {
            listTests.addAll(tests)
        }

        if(new){
            addToSync(this)
            for(h in this.listHomework){
                addToSync(h)
            }
            for(w in this.listWebs){
                addToSync(w)
            }
            for(word in this.listWords){
                addToSync(word)
            }
            for(test in this.listTests){
                addToSync(test)
            }
        }
    }

    fun copy(id: String = this.id, name: String = this.name, listWords: MutableList<Word> = this.listWords,
             listWebs: MutableList<Web> = this.listWebs, listHomework: MutableList<HomeWork> = this.listHomework,
            class_id: String = this.class_id, listTests:MutableList<Test> =this.listTests): Student{
        return Student(id, name, listWords, listWebs, listHomework, class_id, false, listTests)
    }

    private fun genericSearch(id: String, list: List<Syncable>): Syncable?{
        for(t in list){
            if(t.getID() == id){
                return t
            }
        }

        return null
    }

    fun addTest(test: Test){
        listTests.add(test)

        addToSync(test)
    }

    fun addWord(word: Word){
        listWords.add(word)
        addToSync(word)
    }

    fun addWord(words: List<Word>){
        for(w in words){
            addWord(w)
        }
    }

    fun updateWord(word: Word){
        listWords.remove(genericSearch(word.id, listWords))
        listWords.add(word)

        addToSync(word, local = true)
    }

    fun updateWord(word: List<Word>){
        for(w in word){
            updateWord(w)
        }
    }

    fun updateHomework(homework: HomeWork){
        listHomework.remove(genericSearch(homework.id, listHomework))
        listHomework.add(homework)

        addToSync(homework)
    }

    fun addWeb(web: Web){
        listWebs.add(web)

        addToSync(web)
    }

    fun removeWeb(web: Web){
        listWebs.remove(genericSearch(web.id, listWebs))
        removeToSync(web)
    }


    fun removeWord(word: Word){
        listWords.remove(genericSearch(word.id, listWords))
        removeToSync(word)
    }

    fun removeWord(words: List<Word>){
        for(w in words){
            removeWord(w)
        }
    }

    fun getHomeworkByHomeworkID(homeworkID: String): HomeWork?{
        for(t in listHomework){
            if(t.homework_id == homeworkID){
                return t
            }
        }

        return null
    }

    fun getHomework(id: String): HomeWork?{
        return genericSearch(id, listHomework) as HomeWork?
    }

    fun getTest(id: String): Test?{
        return genericSearch(id, listTests) as Test?
    }

    fun getWeb(id: String): Web?{
        return genericSearch(id, listWebs) as Web?
    }

    fun getWord(id: String): Word?{
        return genericSearch(id, listWords) as Word?
    }

    override fun getID(): String{
        return id
    }

    override fun getType(): Syncable.Types {
        return Syncable.Types.Student
    }
}