package com.example.vocab.basic

import com.example.vocab.merge

abstract class StudentSync{
    enum class Actions {
        Add,
        Remove
    }

    private var toSync = mutableMapOf<Actions,MutableList<Syncable>>()
    private var toSyncR = mutableMapOf<Actions,MutableList<Syncable>>()

    init{
        toSync[Actions.Remove] = mutableListOf()
        toSync[Actions.Add] = mutableListOf()
        toSyncR[Actions.Remove] = mutableListOf()
        toSyncR[Actions.Add] = mutableListOf()
    }

    protected fun addToSync(obj: Syncable, local: Boolean = false){
        if(obj.getType() in Syncable.Types.values() && toSync[Actions.Add]?.contains(obj) != true){
            toSync.merge(Actions.Add, mutableListOf(obj))
            if(!local) {
                toSyncR.merge(Actions.Add, mutableListOf(obj))
            }
        }
    }

    protected fun removeToSync(obj: Syncable, local: Boolean = false){
        if(obj.getType() in Syncable.Types.values() && toSync[Actions.Remove]?.contains(obj) != true){
            toSync.merge(Actions.Remove, mutableListOf(obj))
            if(!local) {
                toSyncR.merge(Actions.Remove, mutableListOf(obj))
            }
        }
    }

    fun remoteSync(): Map<Actions,List<Syncable>>{
        return toSyncR
    }

    fun localSync(): Map<Actions,List<Syncable>>{
        return toSync
    }

    protected fun setListLocal(local: Map<Actions,List<Syncable>>){
        val toSave = mutableMapOf<Actions, MutableList<Syncable>>()
        for((action, list) in local){
            toSave[action] = list.toMutableList()
        }

        toSync = toSave
    }

    protected fun setListRemote(Remote: Map<Actions,List<Syncable>>){
        val toSave = mutableMapOf<Actions, MutableList<Syncable>>()
        for((action, list) in Remote){
            toSave[action] = list.toMutableList()
        }

        toSyncR = toSave
    }

    fun localSyncDel(){
        toSync.clear()
        toSync[Actions.Remove] = mutableListOf()
        toSync[Actions.Add] = mutableListOf()
    }

    fun remoteSyncDel(){
        toSyncR.clear()
        toSyncR[Actions.Remove] = mutableListOf()
        toSyncR[Actions.Add] = mutableListOf()
    }

    fun isToSyncRemote(): Boolean{
        return toSyncR[Actions.Add]!!.size !=0 || toSyncR[Actions.Remove]!!.size !=0
    }

    fun isToSyncLocal(): Boolean{
        return toSync[Actions.Add]!!.size !=0 || toSync[Actions.Remove]!!.size !=0
    }
}