package com.example.vocab.gui

import androidx.lifecycle.ViewModel
import com.example.vocab.Tools
import java.lang.Exception

abstract class BaseViewModel(): ViewModel() {
    private val newObjects = mutableListOf<BaseItem<*>>()
    private val deletedObjects = mutableListOf<BaseItem<*>>()

    private var objectThatIsWriting: String? = null

    protected abstract fun onSave(newObjects: List<BaseItem<*>>)
    protected abstract fun onDelete(deletedObjects: List<BaseItem<*>>)
    protected abstract fun onCommit(newObjects: List<BaseItem<*>>, deletedObjects: List<BaseItem<*>>)
    protected abstract fun loadData()

    fun requestAccess(): String?{
        return if(objectThatIsWriting == null){
            objectThatIsWriting = Tools.getUUID()
            return objectThatIsWriting
        } else{
            null
        }
    }

    fun save(newObject: BaseItem<*>, obj: String){
        if(obj == objectThatIsWriting) {
            newObjects.add(newObject)
            onSave(listOf(newObject))
        }
        else throw Exception("Denied writing access to fragmentModel")
    }

    fun save(newObjectsLocal: List<BaseItem<*>>, obj: String){
        if(obj == objectThatIsWriting){
            for(obj in newObjectsLocal){
                newObjects.add(obj)
            }

            onSave(newObjectsLocal)
        }
    }

    fun delete(deletedObjects: List<BaseItem<*>>, obj: String){
        if(obj == objectThatIsWriting){
            for(d in deletedObjects){
                if(d in newObjects) newObjects.remove(d)
                else this.deletedObjects.add(d)
            }

            onDelete(deletedObjects)
        }
        else throw Exception("Denied writing access to fragmentModel")
    }

    fun commit(){
        onCommit(newObjects, deletedObjects)
    }

    fun releaseAccess(){
        objectThatIsWriting = null
        newObjects.clear()
        deletedObjects.clear()

        loadData()
    }
}