package com.example.vocab.studentHomework

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.RelativeLayout
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import com.example.vocab.R
import com.example.vocab.Tools
import com.example.vocab.basic.Word
import com.example.vocab.databinding.FragmentHomeworkTableWordsBinding
import com.example.vocab.gui.CustomTextView
import com.example.vocab.homework.HomeworkSettingModel


class HomeworkTableWords(): Fragment() {
    private lateinit var binding: FragmentHomeworkTableWordsBinding
    private var words: List<Word>? = null

    companion object{
        fun newInstance(words: List<Word>): HomeworkTableWords {
            val newObj = HomeworkTableWords()
            newObj.setArguments(words)

            return newObj
        }
    }

    private fun setArguments(words: List<Word>){
        this.words = words
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Inflate the layout for this fragment
        binding = FragmentHomeworkTableWordsBinding.inflate(inflater, container, false)

        val settingModel: HomeworkSettingModel by viewModels()
        val words = words ?: settingModel.listOfWords ?: throw Exception("internal exception")
        settingModel.listOfWords = words

        val space = Tools.dp2Pixels(6, resources)

        binding.wordsTable.background = Tools.getCircleLayout(R.color.white, requireContext(), resources)
        binding.wordsTable.setPadding(space, space, space, space)

        var lineLength = 0
        var prevButton: CustomTextView? = null
        var prevLineButton: CustomTextView? = null

        for(w in words){
            val button = CustomTextView(requireContext(), w.from)
            button.id = View.generateViewId()

            button.background = Tools.getCircleLayout(R.color.magenta, requireContext(), resources)

            val params = RelativeLayout.LayoutParams(
                button.customWidth,
                button.customHeight
            ).apply {
                leftMargin = space
                bottomMargin = space
                topMargin = space
                rightMargin = space
            }

            lineLength += button.customWidth

            val width = Tools.getScreenWidth(resources) - Tools.dp2Pixels(32, resources)

            prevButton?.let{
                if(lineLength <= width) {
                    params.addRule(RelativeLayout.RIGHT_OF, it.id)
                    prevLineButton?.let{
                        params.addRule(RelativeLayout.BELOW, it.id)
                    }
                }
                else{
                    params.addRule(RelativeLayout.BELOW, it.id)
                    lineLength = button.customWidth
                    prevLineButton = prevButton
                }
            }

            prevButton = button

            binding.wordsTable.addView(button, params)
        }

        return binding.root
    }
}